@@ -31,10 +31,6 @@ namespace dxvk {
       AddDirtyBox(nullptr, i);
     }
 
-    if (m_desc.Pool != D3DPOOL_DEFAULT && pSharedHandle) {
-      throw DxvkError("D3D9: Incompatible pool type for texture sharing.");
-    }
-
     if (IsPoolManaged(m_desc.Pool)) {
       SetAllNeedUpload();
     }
@@ -166,8 +162,9 @@ namespace dxvk {
        || (blockSize.Height && (pDesc->Height & (blockSize.Height - 1))))
         return D3DERR_INVALIDCALL;
     }
-    
-    if (FAILED(DecodeMultiSampleType(pDevice->GetDXVKDevice(), pDesc->MultiSample, pDesc->MultisampleQuality, nullptr)))
+
+    VkSampleCountFlagBits sampleCount;
+    if (FAILED(DecodeMultiSampleType(pDevice->GetDXVKDevice(), pDesc->MultiSample, pDesc->MultisampleQuality, &sampleCount)))
       return D3DERR_INVALIDCALL;
 
     // Using MANAGED pool with DYNAMIC usage is illegal
@@ -241,6 +238,10 @@ namespace dxvk {
         return D3DERR_INVALIDCALL;
     }
 
+    // A multisample RT/DS must not be lockable
+    if (pDesc->IsLockable && sampleCount > VK_SAMPLE_COUNT_1_BIT)
+        return D3DERR_INVALIDCALL;
+
     return D3D_OK;
   }
 
@@ -357,7 +358,6 @@ namespace dxvk {
       imageInfo.sharing.mode = (*pSharedHandle == INVALID_HANDLE_VALUE || *pSharedHandle == nullptr)
         ? DxvkSharedHandleMode::Export
         : DxvkSharedHandleMode::Import;
-      imageInfo.sharing.type = VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_KMT_BIT;
       imageInfo.sharing.handle = *pSharedHandle;
       imageInfo.shared = true;
       // TODO: validate metadata?
@@ -556,7 +556,7 @@ namespace dxvk {
       return VK_IMAGE_LAYOUT_GENERAL;
 
     // Otherwise, pick a layout that can be used for reading.
-    return Usage & VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT
+    return (Usage & VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT)
       ? VK_IMAGE_LAYOUT_DEPTH_STENCIL_READ_ONLY_OPTIMAL
       : VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL;
   }
@@ -633,12 +633,14 @@ namespace dxvk {
   Rc<DxvkImageView> D3D9CommonTexture::CreateView(
           UINT                   Layer,
           UINT                   Lod,
-          VkImageUsageFlags      UsageFlags,
-          bool                   Srgb) {    
+          VkImageUsageFlagBits   UsageFlags,
+          VkImageLayout          Layout,
+          bool                   Srgb) {
     DxvkImageViewKey viewInfo;
     viewInfo.format    = m_mapping.ConversionFormatInfo.FormatColor != VK_FORMAT_UNDEFINED
                        ? PickSRGB(m_mapping.ConversionFormatInfo.FormatColor, m_mapping.ConversionFormatInfo.FormatSrgb, Srgb)
                        : PickSRGB(m_mapping.FormatColor, m_mapping.FormatSrgb, Srgb);
+    viewInfo.layout    = Layout;
     viewInfo.aspects   = lookupFormatInfo(viewInfo.format)->aspectMask;
     viewInfo.usage     = UsageFlags;
     viewInfo.viewType  = GetImageViewTypeFromResourceType(m_type, Layer);
@@ -697,10 +699,22 @@ namespace dxvk {
     if (unlikely(m_mapMode == D3D9_COMMON_TEXTURE_MAP_MODE_SYSTEMMEM))
       return;
 
-    m_sampleView.Color = CreateView(AllLayers, Lod, VK_IMAGE_USAGE_SAMPLED_BIT, false);
+    // The backend will ignore the view layout anyway for images
+    // that have GENERAL (or FEEDBACK_LOOP) as their layout.
+    // This will always be the case for images that can be sampled.
+    // So just pick UNDEFINED here.
+    m_sampleView.Color = CreateView(AllLayers, Lod,
+      VK_IMAGE_USAGE_SAMPLED_BIT, VK_IMAGE_LAYOUT_UNDEFINED, false);
 
-    if (IsSrgbCompatible())
-      m_sampleView.Srgb = CreateView(AllLayers, Lod, VK_IMAGE_USAGE_SAMPLED_BIT, true);
+    if (IsSrgbCompatible()) {
+      m_sampleView.Srgb = CreateView(AllLayers, Lod,
+        VK_IMAGE_USAGE_SAMPLED_BIT, VK_IMAGE_LAYOUT_UNDEFINED, true);
+    }
+
+    if (IsDepthStencil() && GetType() != D3DRTYPE_SURFACE) {
+      m_sampleView.DepthReadOnly = CreateView(AllLayers, Lod,
+        VK_IMAGE_USAGE_SAMPLED_BIT, VK_IMAGE_LAYOUT_DEPTH_READ_ONLY_STENCIL_ATTACHMENT_OPTIMAL, false);
+    }
   }
 
 
