@@ -109,6 +109,110 @@ namespace dxvk {
     void*           mapPtr = nullptr;
   };
 
+  struct D3D9TextureSlotTracking {
+    /* Pixel shaders can access 16 textures/samplers.
+     * Then there's 1 dmap texture/sampler.
+     * Vertex shaders can use 4 textures/samplers.
+     * So unless otherwise noted most bitmasks use 21 bits
+     * and each bit is one texture/sampler slot.
+     * See RemapSamplerState(), IsPSSampler(), IsVSSampler() in d3d9_util.h */
+
+    /** Whether the format of the texture currently bound to each slot is a format that gets fetched in comparison mode. */
+    uint32_t depth = 0;
+
+    /** If a depth texture format isn't supported, we fall back to D32F.
+     * We'll need to clamp the reference value if the original format was an unorm format.
+     * This tracks the texture/sampler slots for which this kind of adjusting needs to be done. */
+    uint32_t drefClamp = 0;
+
+    /** Used to store the type of each bound pixel shader texture.
+     * This is used to generate fixed function shader code
+     * and for PS 1.1 shaders which do not provide this information in the shader bytecode.
+     * SM 1.1 and fixed function doesn't allow sampling textures in the VS, so we only need the 16 PS slots.
+     * There's 3 texture types, so every texture/sampler slot uses 2 bits. */
+    uint32_t textureType = 0;
+
+    /** Whether the type of the texture currently bound to each slot matches the texture type that the shader expects */
+    uint32_t mismatchingTextureType = 0;
+
+    /** Whether projected texture lookup is enabled for each texture/sampler slot. This is only used for generating fixed function shaders. */
+    uint32_t projected = 0;
+
+    /** Whether sampler slots whose state has been changed and bindings in the backend need to be updated */
+    uint32_t samplerStateDirty = 0;
+
+    /** Whether Fetch 4 is enabled for a sampler slot.
+     * This just means the application enabled it using the sampler state.
+     * It does not mean Fetch 4 is actually active, as that depends on other factors
+     * such as the sampling mode and the texture format. */
+    uint32_t fetch4SamplerState = 0;
+
+    /** Whether Fetch 4 is active. */
+    uint32_t fetch4 = 0;
+
+    /** Whether the texture bound to a slot has been changed and bindings in the backend need to be updated */
+    uint32_t textureDirty = 0;
+
+    /** Whether the texture bound to a slot has D3DUSAGE_RENDERTARGET */
+    uint32_t rtUsage = 0;
+
+    /** Whether the texture bound to a slot has D3DUSAGE_DEPTHSTENCIL */
+    uint32_t dsUsage = 0;
+
+    /** Whether the texture bound to a slot is also bound as a render target
+     * and the render target is actually used for writing. */
+    uint32_t unresolvableHazardRT = 0;
+
+    /** Whether the texture bound to a slot is also bound as the depth stencil surface
+     * and depth stencil surface is actually used for depth testing. */
+    uint32_t unresolvableHazardDS = 0;
+
+    /** Whether the texture bound to a slot is also bound as a render target */
+    uint32_t hazardRT = 0;
+
+    /** Whether the texture bound to a slot is also bound as the depth stencil view */
+    uint32_t hazardDS = 0;
+
+    /** Whether there's a texture bound to a slot */
+    uint32_t bound = 0;
+
+    /** Whether there's a texture bound to a slot that needs to be uploaded at draw time */
+    uint32_t needsUpload = 0;
+
+    /** Whether there's a texture bound to a slot that needs to have its mip maps generated */
+    uint32_t needsMipGen = 0;
+  };
+
+  struct D3D9RTSlotTracking {
+    /* D3D9 allows rendering to 4 render targets at the same time.
+     * So all RT bit masks only use the first 4 bits. */
+
+    /** Whether a render target is a D3D9Texture rather than just a D3D9Surface.
+      * Textures can be bound for sampling so there can be a feedback loop if this
+      * RT is also bound for sampling. */
+    uint8_t canBeSampled = 0;
+
+    /** Whether the alpha channel of the format of this RT needs to be manually handled
+      * as part of the blend state. */
+    uint8_t hasAlphaSwizzle = 0;
+  };
+
+  struct D3D9VBSlotTracking {
+    /* D3D9 allows using 16 vertex buffers ('streams'). */
+
+    /** Whether there's a vertex buffer bound to the slot */
+    uint16_t bound = 0;
+
+    /** Whether the vertex buffer at each slot needs to be uploaded at draw time */
+    uint16_t needsUpload = 0;
+
+    /** Whether the vertex buffer for each slot gets copied at draw time to act like DrawUP */
+    uint16_t uploadPerDraw = 0;
+
+    /** Whether instancing is enabled for each slot */
+    uint16_t instanced = 0;
+  };
+
   class D3D9DeviceEx final : public ComObjectClamp<IDirect3DDevice9Ex> {
     constexpr static uint32_t DefaultFrameLatency = 3;
     constexpr static uint32_t MaxFrameLatency     = 20;
@@ -518,7 +622,7 @@ namespace dxvk {
     HRESULT STDMETHODCALLTYPE GetIndices(IDirect3DIndexBuffer9** ppIndexData);
 
     HRESULT STDMETHODCALLTYPE CreatePixelShader(
-      const DWORD*                  pFunction, 
+      const DWORD*                  pFunction,
             IDirect3DPixelShader9** ppShader);
 
     HRESULT STDMETHODCALLTYPE SetPixelShader(IDirect3DPixelShader9* pShader);
@@ -653,15 +757,41 @@ namespace dxvk {
       const D3DDISPLAYMODEEX*      pFullscreenDisplayMode,
             IDirect3DSwapChain9**  ppSwapChain);
 
+    /**
+     * @brief Sets the given sampler state
+     *
+     * @param StateSampler Sampler index (according to our internal way of storing samplers)
+     * @param Type Sampler state type to change
+     * @param Value State value
+     */
     HRESULT SetStateSamplerState(
         DWORD               StateSampler,
         D3DSAMPLERSTATETYPE Type,
         DWORD               Value);
 
+    /**
+     * @brief Sets the given sampler texture
+     *
+     * @param StateSampler Sampler index (according to our internal way of storing samplers)
+     * @param pTexture Texture to use
+     */
     HRESULT SetStateTexture(DWORD StateSampler, IDirect3DBaseTexture9* pTexture);
 
+    /**
+     * @brief Sets the transform for the given sampler
+     *
+     * @param idx Sampler index (according to our internal way of storing samplers)
+     * @param pMatrix Transform matrix
+     */
     HRESULT SetStateTransform(uint32_t idx, const D3DMATRIX* pMatrix);
 
+    /**
+     * @brief Sets the fixed function texture processing state
+     *
+     * @param Stage Sampler index (according to our internal way of storing samplers)
+     * @param Type Fixed function texture stage type
+     * @param Value Value for the state
+     */
     HRESULT SetStateTextureStageState(
             DWORD                      Stage,
             D3D9TextureStageStateTypes Type,
@@ -671,13 +801,13 @@ namespace dxvk {
       return m_dxvkDevice->getShaderPipelineStages();
     }
 
-    static DxvkDeviceFeatures GetDeviceFeatures(const Rc<DxvkAdapter>& adapter);
-
     /**
      * \brief Returns whether the Vulkan device supports the required features for ProcessVertices
      */
     bool SupportsSWVP();
 
+    bool SupportsVCacheQuery() const;
+
     bool IsExtended();
 
     HWND GetWindow();
@@ -699,7 +829,7 @@ namespace dxvk {
 
     /**
      * \brief Locks a subresource of an image
-     * 
+     *
      * \param [in] Subresource The subresource of the image to lock
      * \param [out] pLockedBox The returned locked box of the image, containing data ptr and strides
      * \param [in] pBox The region of the subresource to lock. This offsets the returned data ptr
@@ -800,7 +930,8 @@ namespace dxvk {
     void Flush();
     void FlushAndSync9On12();
 
-    void EndFrame();
+    void BeginFrame(Rc<DxvkLatencyTracker> LatencyTracker, uint64_t FrameId);
+    void EndFrame(Rc<DxvkLatencyTracker> LatencyTracker);
 
     void UpdateActiveRTs(uint32_t index);
 
@@ -809,16 +940,48 @@ namespace dxvk {
 
     void UpdateTextureBitmasks(uint32_t index, DWORD combinedUsage);
 
-    void UpdateActiveHazardsRT(uint32_t rtMask);
+    void UpdateActiveHazardsRT(uint32_t rtMask, uint32_t texMask);
 
     void UpdateActiveHazardsDS(uint32_t texMask);
 
-    void MarkRenderHazards();
+    void EmitFeedbackLoopBarriers();
 
     void UpdateActiveFetch4(uint32_t stateSampler);
 
+    /**
+     * @brief Sets the mismatching texture type bits for all samplers if necessary.
+     *
+     * This function will check all samplers the shader uses and set the  set the mismatching texture type bit for the given sampler if it does not
+     * match the texture type expected by the respective shader.
+     *
+     * It will *not* unset the bit if the texture type does match.
+     *
+     * @param stateSampler Sampler index (according to our internal way of storing samplers)
+     */
+
+     /**
+      * @brief Sets the mismatching texture type bits for all samplers if necessary.
+      *
+      * This function will check all samplers the shader uses and set the  set the mismatching texture type bit for the given sampler if it does not
+      * match the texture type expected by the shader.
+      *
+      * @param shader The shader
+      * @param shaderSamplerMask Mask of all samplers that the shader uses (according to our internal way of storing samplers)
+      * @param shaderSamplerOffset First index of the shader's samplers according to our internal way of storing samplers.
+      *                            Used to transform the sampler indices that are relative to the entire pipeline to ones relative to the shader.
+      */
     void UpdateTextureTypeMismatchesForShader(const D3D9CommonShader* shader, uint32_t shaderSamplerMask, uint32_t shaderSamplerOffset);
 
+    /**
+     * @brief Sets the mismatching texture type bit for the given sampler.
+     *
+     * This function will set the mismatching texture type bit for the given sampler if it does not
+     * match the texture type expected by the respective shader.
+     *
+     * It will *not* unset the bit if the texture type does match.
+     *
+     * @param stateSampler Sampler index (according to our internal way of storing samplers)
+     */
     void UpdateTextureTypeMismatchesForTexture(uint32_t stateSampler);
 
     void UploadManagedTexture(D3D9CommonTexture* pResource);
@@ -841,35 +1004,26 @@ namespace dxvk {
 
     void BindViewportAndScissor();
 
-    inline bool IsAlphaToCoverageEnabled() {
-      const bool alphaTest = m_state.renderStates[D3DRS_ALPHATESTENABLE] != 0;
+    bool IsAlphaToCoverageEnabled() const;
 
-      return (m_amdATOC || (m_nvATOC && alphaTest)) && m_flags.test(D3D9DeviceFlag::ValidSampleMask);
-    }
+    inline bool IsNVDepthBoundsTestEnabled () {
+      // NVDB is not supported by D3D8
+      if (unlikely(m_isD3D8Compatible))
+        return false;
 
-    inline bool IsDepthBiasEnabled() {
-      const auto& rs = m_state.renderStates;
-
-      float depthBias            = bit::cast<float>(rs[D3DRS_DEPTHBIAS]);
-      float slopeScaledDepthBias = bit::cast<float>(rs[D3DRS_SLOPESCALEDEPTHBIAS]);
-
-      return depthBias != 0.0f || slopeScaledDepthBias != 0.0f;
+      return m_state.renderStates[D3DRS_ADAPTIVETESS_X] == uint32_t(D3D9Format::NVDB);
     }
 
     inline bool IsAlphaTestEnabled() {
-      return m_state.renderStates[D3DRS_ALPHATESTENABLE] && !IsAlphaToCoverageEnabled();
+      return m_state.renderStates[D3DRS_ALPHATESTENABLE] && !m_atocEnabled;
     }
 
     inline bool IsZTestEnabled() {
       return m_state.renderStates[D3DRS_ZENABLE] && m_state.depthStencil != nullptr;
     }
 
-    inline bool IsClipPlaneEnabled() {
-      return m_state.renderStates[D3DRS_CLIPPLANEENABLE] != 0;
-    }
-
     void BindMultiSampleState();
-    
+
     void BindBlendState();
 
     void BindBlendFactor();
@@ -888,10 +1042,10 @@ namespace dxvk {
 
     template <DxsoProgramType ShaderStage, typename HardwareLayoutType, typename SoftwareLayoutType, typename ShaderType>
     inline void UploadConstantSet(const SoftwareLayoutType& Src, const D3D9ConstantLayout& Layout, const ShaderType& Shader);
-    
+
     template <DxsoProgramType ShaderStage>
     void UploadConstants();
-    
+
     void UpdateClipPlanes();
 
     /**
@@ -932,7 +1086,7 @@ namespace dxvk {
       D3DPRIMITIVETYPE PrimitiveType,
       UINT             PrimitiveCount,
       UINT             InstanceCount);
-    
+
     uint32_t GetInstanceCount() const;
 
     void PrepareDraw(D3DPRIMITIVETYPE PrimitiveType, bool UploadVBOs, bool UploadIBOs);
@@ -992,8 +1146,8 @@ namespace dxvk {
             VkImageLayout            OldLayout,
             VkImageLayout            NewLayout);
 
-    const D3D9ConstantLayout& GetVertexConstantLayout() { return m_vsLayout; }
-    const D3D9ConstantLayout& GetPixelConstantLayout()  { return m_psLayout; }
+    const D3D9ConstantLayout& GetVertexConstantLayout() { return m_consts[DxsoProgramType::VertexShader].layout; }
+    const D3D9ConstantLayout& GetPixelConstantLayout()  { return m_consts[DxsoProgramType::PixelShader].layout; }
 
     void ResetState(D3DPRESENT_PARAMETERS* pPresentationParameters);
     HRESULT ResetSwapChain(D3DPRESENT_PARAMETERS* pPresentationParameters, D3DDISPLAYMODEEX* pFullscreenDisplayMode);
@@ -1118,6 +1272,10 @@ namespace dxvk {
       return DxvkCsChunkRef(chunk, &m_csChunkPool);
     }
 
+    bool Is9On12Device() const {
+      return m_d3d9On12Args.Enable9On12;
+    }
+
   private:
 
     template<bool AllowFlush = true, typename Cmd>
@@ -1143,12 +1301,12 @@ namespace dxvk {
     }
 
     /**
-     * \brief Returns whether the device has been reset and marks it as true.
+     * \brief Queries current reset counter
      * Used for the deferred surface creation workaround.
      * (Device Reset detection for D3D9SwapChainEx::Present)
      */
-    bool IsDeviceReset() {
-      return std::exchange(m_deviceHasBeenReset, false);
+    uint32_t GetResetCounter() {
+      return m_resetCtr;
     }
 
     template <bool Synchronize9On12>
@@ -1224,8 +1382,7 @@ namespace dxvk {
     template <DxsoProgramType  ProgramType,
               D3D9ConstantType ConstantType>
     inline uint32_t DetermineHardwareRegCount() const {
-      const auto& layout = ProgramType == DxsoProgramType::VertexShader
-        ? m_vsLayout : m_psLayout;
+      const auto& layout = m_consts[ProgramType].layout;
 
       switch (ConstantType) {
         default:
@@ -1396,6 +1553,20 @@ namespace dxvk {
         && !m_state.renderTargets[Index]->IsNull();
     }
 
+    inline D3D9ShaderMasks VSShaderMasks() const {
+      return m_state.vertexShader != nullptr
+        ? m_state.vertexShader->GetCommonShader()->GetShaderMask()
+        : D3D9ShaderMasks();
+    }
+
+    inline D3D9ShaderMasks PSShaderMasks() const {
+      return m_state.pixelShader != nullptr
+        ? m_state.pixelShader->GetCommonShader()->GetShaderMask()
+        : FixedFunctionMask;
+    }
+
+    GpuFlushType GetMaxFlushType() const;
+
     Com<D3D9InterfaceEx>            m_parent;
     D3DDEVTYPE                      m_deviceType;
     HWND                            m_window;
@@ -1457,63 +1628,31 @@ namespace dxvk {
     D3D9InputAssemblyState          m_iaState;
 
     D3D9DeviceFlags                 m_flags;
-    // Last state of depth textures. Doesn't update when NULL is bound.
-    // & with m_activeTextures to normalize.
-    uint32_t                        m_instancedData = 0;
 
-    uint32_t                        m_depthTextures = 0;
-    uint32_t                        m_drefClamp = 0;
-    uint32_t                        m_cubeTextures = 0;
-    uint32_t                        m_textureTypes = 0;
-    uint32_t                        m_mismatchingTextureTypes = 0;
-    uint32_t                        m_projectionBitfield  = 0;
+    D3D9TextureSlotTracking         m_textureSlotTracking;
 
-    uint32_t                        m_dirtySamplerStates = 0;
-    uint32_t                        m_dirtyTextures      = 0;
+    D3D9RTSlotTracking              m_rtSlotTracking;
 
-    uint32_t                        m_activeRTsWhichAreTextures : 4;
-    uint32_t                        m_alphaSwizzleRTs : 4;
-    uint32_t                        m_lastHazardsRT   : 4;
-
-    uint32_t                        m_activeTextureRTs       = 0;
-    uint32_t                        m_activeTextureDSs       = 0;
-    uint32_t                        m_activeHazardsRT        = 0;
-    uint32_t                        m_activeHazardsDS        = 0;
-    uint32_t                        m_activeTextures         = 0;
-    uint32_t                        m_activeTexturesToUpload = 0;
-    uint32_t                        m_activeTexturesToGen    = 0;
-
-    uint32_t                        m_activeVertexBuffers                = 0;
-    uint32_t                        m_activeVertexBuffersToUpload        = 0;
-    uint32_t                        m_activeVertexBuffersToUploadPerDraw = 0;
-
-    // m_fetch4Enabled is whether fetch4 is currently enabled
-    // from the application.
-    //
-    // m_fetch4 is whether it should be enabled in the shader
-    // ie. are we in a correct state to use it
-    // (enabled + texture supports it + point sampled)
-    uint32_t                        m_fetch4Enabled = 0;
-    uint32_t                        m_fetch4        = 0;
-
-    uint32_t                        m_lastHazardsDS = 0;
-    uint32_t                        m_lastSamplerTypesFF = 0;
+    D3D9VBSlotTracking              m_vbSlotTracking;
 
     D3D9SpecializationInfo          m_specInfo = D3D9SpecializationInfo();
 
-    D3D9ShaderMasks                 m_vsShaderMasks = D3D9ShaderMasks();
-    D3D9ShaderMasks                 m_psShaderMasks = FixedFunctionMask;
-
     bool                            m_isSWVP;
     bool                            m_isD3D8Compatible;
-    bool                            m_amdATOC          = false;
-    bool                            m_nvATOC           = false;
     bool                            m_ffZTest          = false;
-    
+
+    // the enablement of below features is tracked independently
+    // of render states both due to complexity and to avoid
+    // incurring overhead on all render state changes
+    bool                            m_alphaTestEnabled = false;
+    // vendor hack state tracking
+    bool                            m_atocEnabled      = false;
+    bool                            m_nvdbEnabled      = false;
+
     VkImageLayout                   m_hazardLayout = VK_IMAGE_LAYOUT_GENERAL;
 
     bool                            m_usingGraphicsPipelines = false;
-    bool                            m_deviceHasBeenReset = false;
+    uint32_t                        m_resetCtr = 0u;
 
     DxvkDepthBiasRepresentation     m_depthBiasRepresentation = { VK_DEPTH_BIAS_REPRESENTATION_LEAST_REPRESENTABLE_VALUE_FORMAT_EXT, false };
     float                           m_depthBiasScale  = 0.0f;
@@ -1521,18 +1660,9 @@ namespace dxvk {
     uint32_t                        m_robustSSBOAlignment     = 1;
     uint32_t                        m_robustUBOAlignment      = 1;
 
-    uint32_t                        m_vsFloatConstsCount = 0;
-    uint32_t                        m_vsIntConstsCount   = 0;
-    uint32_t                        m_vsBoolConstsCount  = 0;
-    uint32_t                        m_psFloatConstsCount = 0;
-    VkDeviceSize                    m_boundVSConstantsBufferSize = 0;
-    VkDeviceSize                    m_boundPSConstantsBufferSize = 0;
-
-    D3D9ConstantLayout              m_vsLayout;
-    D3D9ConstantLayout              m_psLayout;
     D3D9ConstantSets                m_consts[DxsoProgramTypes::Count];
-	
-	D3D9UserDefinedAnnotation*      m_annotation = nullptr;
+
+    D3D9UserDefinedAnnotation*      m_annotation = nullptr;
 
     D3D9ViewportInfo                m_viewportInfo;
 
@@ -1565,6 +1695,7 @@ namespace dxvk {
     Direct3DState9                  m_state;
 
     D3D9VkInteropDevice             m_d3d9Interop;
+    D3D9ON12_ARGS                   m_d3d9On12Args = { };
     D3D9On12                        m_d3d9On12;
     DxvkD3D8Bridge                  m_d3d8Bridge;
 
