@@ -1,4 +1,5 @@
 #include "d3d9_format.h"
+#include "d3d9_adapter.h"
 
 namespace dxvk {
 
@@ -175,7 +176,7 @@ namespace dxvk {
         { VK_COMPONENT_SWIZZLE_R, VK_COMPONENT_SWIZZLE_G,
           VK_COMPONENT_SWIZZLE_B, VK_COMPONENT_SWIZZLE_ONE },
         { D3D9ConversionFormat_W11V11U10,
-        // can't use B10G11R11 bc this is a snorm type
+        // can't use B10G11R11 because this is a snorm type
           VK_FORMAT_R16G16B16A16_SNORM } };
 
       case D3D9Format::UYVY: return {
@@ -267,15 +268,9 @@ namespace dxvk {
         VK_FORMAT_UNDEFINED,
         VK_IMAGE_ASPECT_DEPTH_BIT | VK_IMAGE_ASPECT_STENCIL_BIT };
 
-      case D3D9Format::D32_LOCKABLE: return {
-        VK_FORMAT_D32_SFLOAT,
-        VK_FORMAT_UNDEFINED,
-        VK_IMAGE_ASPECT_DEPTH_BIT };
+      case D3D9Format::D32_LOCKABLE: return {}; // Unsupported (everywhere)
 
-      case D3D9Format::S8_LOCKABLE: return {
-        VK_FORMAT_S8_UINT,
-        VK_FORMAT_UNDEFINED,
-        VK_IMAGE_ASPECT_STENCIL_BIT };
+      case D3D9Format::S8_LOCKABLE: return {}; // Unsupported (everywhere)
 
       case D3D9Format::L16: return {
         VK_FORMAT_R16_UNORM,
@@ -348,10 +343,7 @@ namespace dxvk {
 
       case D3D9Format::A1: return {}; // Unsupported
 
-      case D3D9Format::A2B10G10R10_XR_BIAS: return {
-        VK_FORMAT_A2B10G10R10_SNORM_PACK32,
-        VK_FORMAT_UNDEFINED,
-        VK_IMAGE_ASPECT_COLOR_BIT };
+      case D3D9Format::A2B10G10R10_XR_BIAS: return {}; // Unsupported (everywhere)
 
       case D3D9Format::BINARYBUFFER: return {
         VK_FORMAT_R8_UINT,
@@ -402,6 +394,8 @@ namespace dxvk {
 
       case D3D9Format::ATOC: return {}; // Driver hack, handled elsewhere
 
+      case D3D9Format::SSAA: return {}; // Driver hack, handled elsewhere
+
       case D3D9Format::INTZ: return {
         VK_FORMAT_D24_UNORM_S8_UINT,
         VK_FORMAT_UNDEFINED,
@@ -429,9 +423,28 @@ namespace dxvk {
 
       case D3D9Format::RAWZ: return {}; // Unsupported
 
-      case D3D9Format::R16:  return {}; // Unsupported
+      // EXT1, FXT1, GXT1 and HXT1 are checked for support
+      // by D3D9 SAGE engine games (e.g. Command & Conquer 3)
 
-      case D3D9Format::AL16: return {}; // Unsupported
+      case D3D9Format::EXT1: return {}; // Unsupported (everywhere)
+
+      case D3D9Format::FXT1: return {}; // Unsupported (everywhere)
+
+      case D3D9Format::GXT1: return {}; // Unsupported (everywhere)
+
+      case D3D9Format::HXT1: return {}; // Unsupported (everywhere)
+
+      // AL16 and R16 FOURCCs are often checked for support by
+      // various D3D8 and early D3D9 games. AR16 and L16 (FOURCC)
+      // are also checked for support, but to a lesser extent.
+
+      case D3D9Format::AL16: return {}; // Unsupported (everywhere)
+
+      case D3D9Format::AR16: return {}; // Unsupported (everywhere)
+
+      case D3D9Format::R16:  return {}; // Unsupported (everywhere)
+
+      case D3D9Format::L16_FOURCC:  return {}; // Unsupported (everywhere)
 
       default:
         Logger::warn(str::format("ConvertFormat: Unknown format encountered: ", Format));
@@ -461,17 +474,21 @@ namespace dxvk {
   }
 
   D3D9VkFormatTable::D3D9VkFormatTable(
-    const Rc<DxvkAdapter>& adapter,
-    const D3D9Options&     options) {
+            D3D9Adapter*     pParent,
+      const Rc<DxvkAdapter>& adapter,
+      const D3D9Options&     options)
+    : m_parent (pParent) {
 
-    const auto& props = adapter->deviceProperties();
-    uint32_t vendorId  = options.customVendorId == -1 ? props.vendorID : uint32_t(options.customVendorId);
+    const uint32_t vendorId = m_parent->GetVendorId();
+    const bool     isNvidia = vendorId == uint32_t(DxvkGpuVendor::Nvidia);
+    const bool     isAmd    = vendorId == uint32_t(DxvkGpuVendor::Amd);
+    const bool     isIntel  = vendorId == uint32_t(DxvkGpuVendor::Intel);
 
     // NVIDIA does not natively support any DF formats
-    m_dfSupport = vendorId == uint32_t(DxvkGpuVendor::Nvidia) ? false : options.supportDFFormats;
+    m_dfSupport = !isNvidia ? options.supportDFFormats : false;
     m_x4r4g4b4Support = options.supportX4R4G4B4;
     // Only AMD supports D16_LOCKABLE natively
-    m_d16lockableSupport = vendorId == uint32_t(DxvkGpuVendor::Amd) ? true : options.supportD16Lockable;
+    m_d16lockableSupport = isAmd;
 
     // AMD do not support 24-bit depth buffers on Vulkan,
     // so we have to fall back to a 32-bit depth format.
@@ -486,6 +503,11 @@ namespace dxvk {
       VK_FORMAT_FEATURE_2_DEPTH_STENCIL_ATTACHMENT_BIT |
       VK_FORMAT_FEATURE_2_SAMPLED_IMAGE_BIT);
 
+    // AMD and Nvidia support D32F_LOCKABLE natively (Intel does not)
+    m_d32flockableSupport = !isIntel;
+    // Only Intel supports D24FS8 natively
+    m_d24fs8Support = isIntel;
+
     if (!m_d24s8Support)
       Logger::info("D3D9: VK_FORMAT_D24_UNORM_S8_UINT -> VK_FORMAT_D32_SFLOAT_S8_UINT");
 
@@ -504,6 +526,10 @@ namespace dxvk {
     if (Format == D3D9Format::X4R4G4B4 && !m_x4r4g4b4Support)
       return D3D9_VK_FORMAT_MAPPING();
 
+    // W11V11U10 is only supported by d3d8
+    if (Format == D3D9Format::W11V11U10 && !m_parent->IsD3D8Compatible())
+      return D3D9_VK_FORMAT_MAPPING();
+
     if (Format == D3D9Format::D16_LOCKABLE && !m_d16lockableSupport)
       return D3D9_VK_FORMAT_MAPPING();
 
@@ -512,9 +538,15 @@ namespace dxvk {
 
     if (Format == D3D9Format::DF24 && !m_dfSupport)
       return D3D9_VK_FORMAT_MAPPING();
-    
+
+    if (Format == D3D9Format::D32F_LOCKABLE && !m_d32flockableSupport)
+      return D3D9_VK_FORMAT_MAPPING();
+
+    if (Format == D3D9Format::D24FS8 && !m_d24fs8Support)
+      return D3D9_VK_FORMAT_MAPPING();
+
     if (!m_d24s8Support && mapping.FormatColor == VK_FORMAT_D24_UNORM_S8_UINT)
-      mapping.FormatColor = mapping.Aspect & VK_IMAGE_ASPECT_STENCIL_BIT ? VK_FORMAT_D32_SFLOAT_S8_UINT : VK_FORMAT_D32_SFLOAT;
+      mapping.FormatColor = (mapping.Aspect & VK_IMAGE_ASPECT_STENCIL_BIT) ? VK_FORMAT_D32_SFLOAT_S8_UINT : VK_FORMAT_D32_SFLOAT;
 
     if (!m_d16s8Support && mapping.FormatColor == VK_FORMAT_D16_UNORM_S8_UINT)
       mapping.FormatColor = m_d24s8Support ? VK_FORMAT_D24_UNORM_S8_UINT : VK_FORMAT_D32_SFLOAT_S8_UINT;
@@ -525,13 +557,21 @@ namespace dxvk {
 
   const DxvkFormatInfo* D3D9VkFormatTable::GetUnsupportedFormatInfo(
     D3D9Format            Format) const {
-    static const DxvkFormatInfo r8b8g8      = { 3, VK_IMAGE_ASPECT_COLOR_BIT };
-    static const DxvkFormatInfo r3g3b2      = { 1, VK_IMAGE_ASPECT_COLOR_BIT };
-    static const DxvkFormatInfo a8r3g3b2    = { 2, VK_IMAGE_ASPECT_COLOR_BIT };
-    static const DxvkFormatInfo a8p8        = { 2, VK_IMAGE_ASPECT_COLOR_BIT };
-    static const DxvkFormatInfo p8          = { 1, VK_IMAGE_ASPECT_COLOR_BIT };
-    static const DxvkFormatInfo cxv8u8      = { 2, VK_IMAGE_ASPECT_COLOR_BIT };
-    static const DxvkFormatInfo unknown     = {};
+    static const DxvkFormatInfo r8b8g8        = { 3, VK_IMAGE_ASPECT_COLOR_BIT };
+    static const DxvkFormatInfo r3g3b2        = { 1, VK_IMAGE_ASPECT_COLOR_BIT };
+    static const DxvkFormatInfo x4r4g4b4      = { 2, VK_IMAGE_ASPECT_COLOR_BIT };
+    static const DxvkFormatInfo a8r3g3b2      = { 2, VK_IMAGE_ASPECT_COLOR_BIT };
+    static const DxvkFormatInfo a8p8          = { 2, VK_IMAGE_ASPECT_COLOR_BIT };
+    static const DxvkFormatInfo p8            = { 1, VK_IMAGE_ASPECT_COLOR_BIT };
+    static const DxvkFormatInfo w11v11u10     = { 4, VK_IMAGE_ASPECT_COLOR_BIT };
+    static const DxvkFormatInfo cxv8u8        = { 2, VK_IMAGE_ASPECT_COLOR_BIT };
+    // Unsupported and potentially unsupported lockable depth formats need to be
+    // listed here in order to allow for the creation of offscreen plain surfaces
+    static const DxvkFormatInfo d16_lockable  = { 2, VK_IMAGE_ASPECT_DEPTH_BIT };
+    static const DxvkFormatInfo d32f_lockable = { 4, VK_IMAGE_ASPECT_DEPTH_BIT };
+    static const DxvkFormatInfo d32_lockable  = { 4, VK_IMAGE_ASPECT_DEPTH_BIT };
+    static const DxvkFormatInfo s8_lockable   = { 1, VK_IMAGE_ASPECT_STENCIL_BIT };
+    static const DxvkFormatInfo unknown       = {};
 
     switch (Format) {
       case D3D9Format::R8G8B8:
@@ -540,6 +580,10 @@ namespace dxvk {
       case D3D9Format::R3G3B2:
         return &r3g3b2;
 
+      // potentially unsupported through a config option
+      case D3D9Format::X4R4G4B4:
+        return &x4r4g4b4;
+
       case D3D9Format::A8R3G3B2:
         return &a8r3g3b2;
 
@@ -549,6 +593,10 @@ namespace dxvk {
       case D3D9Format::P8:
         return &p8;
 
+      // only supported by d3d8
+      case D3D9Format::W11V11U10:
+        return &w11v11u10;
+
       // MULTI2_ARGB8 -> Don't have a clue what this is.
 
       case D3D9Format::CxV8U8:
@@ -558,18 +606,40 @@ namespace dxvk {
       // Gonna just pretend that doesn't exist until something
       // depends on that.
 
+      // only supported on AMD
+      case D3D9Format::D16_LOCKABLE:
+        return &d16_lockable;
+
+      // unsupported on Intel
+      case D3D9Format::D32F_LOCKABLE:
+        return &d32f_lockable;
+
+      // only considered on d3d9Ex interfaces
+      case D3D9Format::D32_LOCKABLE:
+        if (m_parent->IsExtended())
+          return &d32_lockable;
+
+        [[fallthrough]];
+
+      // only considered on d3d9Ex interfaces
+      case D3D9Format::S8_LOCKABLE:
+        if (m_parent->IsExtended())
+          return &s8_lockable;
+
+        [[fallthrough]];
+
       default:
         return &unknown;
     }
   }
-  
+
 
   bool D3D9VkFormatTable::CheckImageFormatSupport(
     const Rc<DxvkAdapter>&      Adapter,
           VkFormat              Format,
-          VkFormatFeatureFlags2  Features) const {
+          VkFormatFeatureFlags2 Features) const {
     DxvkFormatFeatures supported = Adapter->getFormatFeatures(Format);
-    
+
     return (supported.linear  & Features) == Features
         || (supported.optimal & Features) == Features;
   }
