@@ -11,6 +11,10 @@ namespace dxvk {
   class D3D9VertexDecl;
   class D3D9DeviceEx;
 
+  struct D3D9SwvpEmuArgs {
+    uint32_t exportOffset;
+  };
+
   struct D3D9CompactVertexElement {
       uint16_t Stream : 4;
       uint16_t Type : 5;
@@ -40,6 +44,10 @@ namespace dxvk {
 
     Rc<DxvkShader> GetShaderModule(D3D9DeviceEx* pDevice,  D3D9CompactVertexElements&& elements);
 
+    UINT GetShaderCount() const {
+      return m_modules.size();
+    }
+
   private:
 
     dxvk::mutex                               m_mutex;
