@@ -102,10 +102,11 @@ namespace dxvk {
   struct DxsoSamplerInfo {
     uint32_t dimensions = 0;
 
-    uint32_t varId = 0;
-    uint32_t typeId = 0;
-
+    uint32_t imageVarId = 0;
     uint32_t imageTypeId = 0;
+
+    uint32_t sampledTypeId = 0u;
+    uint32_t samplerIndex = 0u;
   };
 
   enum DxsoSamplerType : uint32_t {
@@ -259,7 +260,7 @@ namespace dxvk {
 
     DxsoShaderMetaInfo         m_meta;
     DxsoDefinedConstants       m_constants;
-    uint32_t                   m_maxDefinedConstant;
+    uint32_t                   m_maxDefinedConstant = 0u;
 
     SpirvModule                m_module;
 
@@ -279,7 +280,7 @@ namespace dxvk {
     // Predicate registers
     std::array<
       DxsoRegisterPointer,
-      1> m_pRegs;
+      1> m_pRegs = { };
 
     //////////////////////////////////////////////////////////////////
     // Array of input values. Since v# and o# registers are indexable
@@ -313,7 +314,7 @@ namespace dxvk {
     // Working tex/coord registers (PS)
     std::array<
       DxsoRegisterPointer,
-      DxsoMaxTextureRegs> m_tRegs;
+      DxsoMaxTextureRegs> m_tRegs = { };
 
     ///////////////////////////////////////////////
     // Control flow information. Stores labels for
@@ -327,7 +328,7 @@ namespace dxvk {
 
     ////////////
     // Samplers
-    std::array<DxsoSampler, 17> m_samplers;
+    std::array<DxsoSampler, 17> m_samplers = { };
 
     ////////////////////////////////////////////
     // What io regswe need to
@@ -364,8 +365,14 @@ namespace dxvk {
     uint32_t m_specUbo = 0;
 
     uint32_t m_rsBlock = 0;
+    uint32_t m_rsFirstSampler = 0u;
+
+    uint32_t m_samplerArray = 0u;
+
     uint32_t m_mainFuncLabel = 0;
 
+    DxvkPushDataBlock m_samplerPushData;
+
     //////////////////////////////////////
     // Common function definition methods
     void emitInit();
@@ -681,7 +688,7 @@ namespace dxvk {
     void emitInputSetup();
 
     void emitVsClipping();
-    void setupRenderStateInfo();
+    void setupRenderStateInfo(uint32_t samplerCount);
     void emitFog();
     void emitPsProcessing();
     void emitOutputDepthClamp();
@@ -712,4 +719,4 @@ namespace dxvk {
 
   };
 
-}
\ No newline at end of file
+}
