@@ -15,7 +15,6 @@
 namespace dxvk {
   
   class DxvkDevice;
-  class DxvkStateCache;
   class DxvkPipelineManager;
   class DxvkPipelineWorkers;
 
@@ -31,6 +30,7 @@ namespace dxvk {
     HasStorageDescriptors,
     HasSampleRateShading,
     HasSampleMaskExport,
+    UnrollMergedDraws,
   };
 
   using DxvkGraphicsPipelineFlags = Flags<DxvkGraphicsPipelineFlag>;
@@ -49,7 +49,7 @@ namespace dxvk {
     DxvkGraphicsPipelineVertexInputState(
       const DxvkDevice*                     device,
       const DxvkGraphicsPipelineStateInfo&  state,
-      const DxvkShader*                     vs);
+      const DxvkGraphicsPipelineShaders&    shaders);
 
     VkPipelineInputAssemblyStateCreateInfo          iaInfo        = { VK_STRUCTURE_TYPE_PIPELINE_INPUT_ASSEMBLY_STATE_CREATE_INFO };
     VkPipelineVertexInputStateCreateInfo            viInfo        = { VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_STATE_CREATE_INFO };
@@ -108,7 +108,7 @@ namespace dxvk {
     DxvkGraphicsPipelineFragmentOutputState(
       const DxvkDevice*                     device,
       const DxvkGraphicsPipelineStateInfo&  state,
-      const DxvkShader*                     fs);
+      const DxvkGraphicsPipelineShaders&    shaders);
 
     VkPipelineRenderingCreateInfo                   rtInfo = { VK_STRUCTURE_TYPE_PIPELINE_RENDERING_CREATE_INFO };
     VkPipelineColorBlendStateCreateInfo             cbInfo = { VK_STRUCTURE_TYPE_PIPELINE_COLOR_BLEND_STATE_CREATE_INFO };
@@ -169,9 +169,7 @@ namespace dxvk {
     DxvkGraphicsPipelinePreRasterizationState(
       const DxvkDevice*                     device,
       const DxvkGraphicsPipelineStateInfo&  state,
-      const DxvkShader*                     tes,
-      const DxvkShader*                     gs,
-      const DxvkShader*                     fs);
+      const DxvkGraphicsPipelineShaders&    shaders);
 
     VkPipelineViewportStateCreateInfo                     vpInfo              = { VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_STATE_CREATE_INFO };
     VkPipelineTessellationStateCreateInfo                 tsInfo              = { VK_STRUCTURE_TYPE_PIPELINE_TESSELLATION_STATE_CREATE_INFO };
@@ -186,9 +184,8 @@ namespace dxvk {
     size_t hash() const;
 
     static bool isLineRendering(
-      const DxvkGraphicsPipelineStateInfo&  state,
-      const DxvkShader*                     tes,
-      const DxvkShader*                     gs);
+      const DxvkGraphicsPipelineShaders&    shaders,
+      const DxvkGraphicsPipelineStateInfo&  state);
 
   };
 
@@ -227,7 +224,7 @@ namespace dxvk {
             DxvkGraphicsPipelineFlags       flags);
 
     VkPipelineDynamicStateCreateInfo  dyInfo    = { VK_STRUCTURE_TYPE_PIPELINE_DYNAMIC_STATE_CREATE_INFO };
-    std::array<VkDynamicState, 12>    dyStates  = { };
+    std::array<VkDynamicState, 20>    dyStates  = { };
 
     bool eq(const DxvkGraphicsPipelineDynamicState& other) const;
 
@@ -347,6 +344,16 @@ namespace dxvk {
   };
 
 
+  /**
+   * \brief Graphics pipeline handle
+   */
+  struct DxvkGraphicsPipelineHandle {
+    VkPipeline                handle      = VK_NULL_HANDLE;
+    DxvkGraphicsPipelineType  type        = DxvkGraphicsPipelineType::FastPipeline;
+    DxvkAttachmentMask        attachments = { };
+  };
+
+
   /**
    * \brief Graphics pipeline instance
    * 
@@ -358,16 +365,35 @@ namespace dxvk {
     DxvkGraphicsPipelineInstance(
       const DxvkGraphicsPipelineStateInfo&  state_,
             VkPipeline                      baseHandle_,
-            VkPipeline                      fastHandle_)
+            VkPipeline                      fastHandle_,
+            DxvkAttachmentMask              attachments_)
     : state       (state_),
       baseHandle  (baseHandle_),
       fastHandle  (fastHandle_),
-      isCompiling (fastHandle_ != VK_NULL_HANDLE) { }
+      isCompiling (fastHandle_ != VK_NULL_HANDLE),
+      attachments (attachments_) { }
 
     DxvkGraphicsPipelineStateInfo state;
     std::atomic<VkPipeline>       baseHandle  = { VK_NULL_HANDLE };
     std::atomic<VkPipeline>       fastHandle  = { VK_NULL_HANDLE };
     std::atomic<VkBool32>         isCompiling = { VK_FALSE };
+    DxvkAttachmentMask            attachments = { };
+
+    DxvkGraphicsPipelineHandle getHandle() const {
+      // Find a pipeline handle to use. If no optimized pipeline has
+      // been compiled yet, use the slower base pipeline instead.
+      DxvkGraphicsPipelineHandle result;
+      result.handle = fastHandle.load(std::memory_order_acquire);
+      result.type = DxvkGraphicsPipelineType::FastPipeline;
+      result.attachments = attachments;
+
+      if (likely(fastHandle))
+        return result;
+
+      result.handle = baseHandle.load(std::memory_order_acquire);
+      result.type = DxvkGraphicsPipelineType::BasePipeline;
+      return result;
+    }
   };
 
 
@@ -412,10 +438,10 @@ namespace dxvk {
             uint32_t                          specConstantMask)
     : shState(shaders, state),
       dyState(device, state, flags),
-      viState(device, state, shaders.vs.ptr()),
-      prState(device, state, shaders.tes.ptr(), shaders.gs.ptr(), shaders.fs.ptr()),
+      viState(device, state, shaders),
+      prState(device, state, shaders),
       fsState(device, state),
-      foState(device, state, shaders.fs.ptr()),
+      foState(device, state, shaders),
       scState(specConstantMask, state.sc) { }
 
     DxvkGraphicsPipelineShaderState           shState;
@@ -465,7 +491,6 @@ namespace dxvk {
             DxvkDevice*                 device,
             DxvkPipelineManager*        pipeMgr,
             DxvkGraphicsPipelineShaders shaders,
-            DxvkBindingLayoutObjects*   layout,
             DxvkShaderPipelineLibrary*  vsLibrary,
             DxvkShaderPipelineLibrary*  fsLibrary);
 
@@ -488,15 +513,11 @@ namespace dxvk {
     }
     
     /**
-     * \brief Pipeline layout
-     * 
-     * Stores the pipeline layout and the descriptor set
-     * layout, as well as information on the resource
-     * slots used by the pipeline.
+     * \brief Queries pipeline layout
      * \returns Pipeline layout
      */
-    DxvkBindingLayoutObjects* getBindings() const {
-      return m_bindings;
+    const DxvkPipelineBindings* getLayout() const {
+      return &m_layout;
     }
 
     /**
@@ -531,7 +552,7 @@ namespace dxvk {
      * \param [in] state Pipeline state vector
      * \returns Pipeline handle and handle type
      */
-    std::pair<VkPipeline, DxvkGraphicsPipelineType> getPipelineHandle(
+    DxvkGraphicsPipelineHandle getPipelineHandle(
       const DxvkGraphicsPipelineStateInfo&    state);
     
     /**
@@ -577,11 +598,10 @@ namespace dxvk {
     DxvkDevice*                 m_device;    
     DxvkPipelineManager*        m_manager;
     DxvkPipelineWorkers*        m_workers;
-    DxvkStateCache*             m_stateCache;
     DxvkPipelineStats*          m_stats;
 
     DxvkGraphicsPipelineShaders m_shaders;
-    DxvkBindingLayoutObjects*   m_bindings;
+    DxvkPipelineBindings        m_layout;
     DxvkGlobalPipelineBarrier   m_barrier;
     DxvkGraphicsPipelineFlags   m_flags;
 
@@ -645,13 +665,15 @@ namespace dxvk {
     
     uint32_t computeSpecConstantMask() const;
 
+    DxvkAttachmentMask computeAttachmentMask(
+      const DxvkGraphicsPipelineStateInfo& state) const;
+
     bool validatePipelineState(
       const DxvkGraphicsPipelineStateInfo& state,
             bool                           trusted) const;
-    
-    void writePipelineStateToCache(
-      const DxvkGraphicsPipelineStateInfo& state) const;
-    
+
+    DxvkPipelineLayoutBuilder buildPipelineLayout() const;
+
     void logPipelineState(
             LogLevel                       level,
       const DxvkGraphicsPipelineStateInfo& state) const;
@@ -660,4 +682,4 @@ namespace dxvk {
 
   };
   
-}
\ No newline at end of file
+}
